from PySide6.QtWidgets import(QApplication, QMainWindow, QWidget,QGridLayout,
                            QComboBox, QLineEdit, QPushButton,QListWidget,
                            QScrollArea, QLabel, QHBoxLayout, QFrame)
from PySide6.QtGui import QPixmap, QImage, QIcon
from PySide6.QtCore import Qt
from data import Data
import requests

class Page:
    def __init__(self, frame, detail): 
        frame.setSpacing(10)
        row = 0

        url = detail.get('cover url')
        image = QImage(url)
        image.loadFromData(requests.get(url).content)
        label = QLabel()
        label.setAlignment(Qt.AlignmentFlag.AlignTop)
        pixmap = QPixmap(image)
        label.setPixmap(pixmap.scaled(150,220))

        frame.addWidget(label, 0,0,5,1)

        for key, value in detail.items():
            if key != 'cover url' and key != 'plot outline' and key != 'cast':
                label = QLabel(key.title())
                label.setAlignment(Qt.AlignmentFlag.AlignTop)
                label.setStyleSheet('font-weight: 600; margin-top:1em;')
                frame.addWidget(label, row,1,1,1)

                if key == 'cast':
                    value = ', '.join(value)
                label = QLabel(str(value))
                label.setAlignment(Qt.AlignmentFlag.AlignTop)
                label.setStyleSheet('margin-top: 1em;')
                label.setWordWrap(True)
                frame.addWidget(label, row, 2, 1, 1)
                row += 1
        frame.setRowStretch(5, 1)
        frame.setColumnStretch(3,1)
        
        label = QLabel('Plot Outline:')
        label.setStyleSheet('font-weight: bold;')
        frame.addWidget(label, 6, 0, 1, 1)

        label = QLabel(detail.get('plot outline'))
        label.setWordWrap(True)
        frame.addWidget(label, 7,0,1,5)

        label = QLabel('Cast:')
        label.setStyleSheet('font-weight: bold;')
        frame.addWidget(label,8,0,1,1)

        label = QLabel(', '.join(detail.get('cast')))
        label.setWordWrap(True)
        frame.addWidget(label, 9,0,1,5)

        frame.setRowStretch(10,1)
        

class Window(QMainWindow):
    ''' Window class is for visual display '''
    def __init__(self):
        super().__init__()

        # Main window setup
        container = QGridLayout()
        

        widget = QWidget()
        widget.setLayout(container)
        self.setCentralWidget(widget)
        self.show()
        
        app = QApplication.instance()
        screen = app.primaryScreen()
        screen_size = screen.geometry()
        width = screen_size.width()
        height = screen_size.height()
        self.setFixedSize(width/2, height/2)

        # Icon for taskbar
        url = 'my-python-logo.png'
        img = QPixmap(url)
        self.setWindowIcon(QIcon(img))

        # Create the header
        header = QLabel('PySide6 Movie/Actor/Actress Search')
        header.setStyleSheet('background-color: #ffa500; color: #ffffff; font-size: 40px; \
                            font-weight: bold; font-family: cursive; padding: 10px;')
        header.setAlignment(Qt.AlignmentFlag.AlignCenter)

        # Add header to container
        container.addWidget(header, 0,0,1,1)
        

        # Create hbox container to hold to row items
        searchbar = QHBoxLayout()
        searchbar_frame = QFrame()
        searchbar_frame.setLayout(searchbar)
        searchbar_frame.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        container.addWidget(searchbar_frame, 1,0,1,1)

        # Create dropbox prefix
        label = QLabel('Search By:')
        label.setStyleSheet('font-weight: 600;')
        searchbar.addWidget(label)

        # Create dropbox
        self.dropbox = QComboBox()
        self.dropbox.addItems(('Person', 'Movie'))
        searchbar.addWidget(self.dropbox)

        # Create entry field
        self.term = QLineEdit()
        self.term.setPlaceholderText('Enter actor/actress name')
        searchbar.addWidget(self.term)

        # Create search button
        self.btn = QPushButton('Search')
        self.btn.setCursor(Qt.CursorShape(13))
        searchbar.addWidget(self.btn)

        # Container for listboxes
        mid_container = QHBoxLayout()
        container.addLayout(mid_container, 2,0,1,1)

        self.leftbox = QListWidget()
        self.leftbox.setSpacing(5)

        mid_container.addWidget(self.leftbox)

        # Create right window panel
        self.frame_container = QGridLayout()
        frame = QFrame()
        frame.setLayout(self.frame_container)
        frame.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        mid_container.addWidget(frame)

        self.scrollbar = QScrollArea(widget)
        self.scrollbar.setWidget(frame)
        self.scrollbar.setWidgetResizable(True)
        
        mid_container.addWidget(self.scrollbar)
        mid_container.setStretch(0,1)
        mid_container.setStretch(1,1)
        
        # Create footer container
        footerframe = QFrame()
        footerframe.setStyleSheet('background-color: #ffa500;')
        footerbox = QGridLayout()
        footerframe.setLayout(footerbox)
        footerframe.setFrameStyle(QFrame.StyledPanel | QFrame.Plain)
        footerbox.setContentsMargins(0,0,35,0)

        footerbox.setColumnStretch(1, 1)
        footerbox.setColumnStretch(6,1)

        container.addWidget(footerframe, 3,0,1,1)

        label = QLabel('my-python ©')
        footerbox.addWidget(label, 0,2,1,1)

        footerbox.addWidget(QLabel(), 0,3,1,1)

        #Logo image
        label = QLabel()
        pixmap = QPixmap('my-python-logo.png')
        pixmap = pixmap.scaled(pixmap.width()/5, pixmap.height()/5)
        label.setPixmap(pixmap)

        footerbox.addWidget(label, 0,4,1,1)

        footerbox.addWidget(QLabel('™'), 0,5,1,1)

class Controller:
    ''' Controller class is for communication
        between the classes window and data '''
    def __init__(self, data, window):
        self.data = data
        self.window = window
        
        # Detect dropbox change
        self.window.dropbox.currentTextChanged.connect(self.dropbox_change)

        # Search buttonclick
        self.window.btn.clicked.connect(self.populate)

        # Setup to use keyboard enter button
        self.window.term.returnPressed.connect(self.populate)

        # Display selected movie information
        self.window.leftbox.clicked.connect(self.display)

    def dropbox_change(self):
        ''' Method detect change in search item person or movie '''
        self.window.leftbox.clear()
        if self.window.dropbox.currentText() == 'Person':
            self.window.term.setPlaceholderText('Search for actor/actress name')
        elif self.window.dropbox.currentText() == 'Movie':
            self.window.term.setPlaceholderText('Search for a movie')

    def populate(self):
        ''' Method populates the left window '''
        term = self.window.term.text().strip()
        parm = self.window.dropbox.currentText()

        if term:
            self.window.term.clear()
            self.window.leftbox.clear()
            self.keys = {}
            if parm == 'Person':
                person_id = self.data.search_person(term).personID
                person = self.data.get_person(person_id)

                for id, name in person.items():
                    self.window.leftbox.addItem(str(name))
                    self.keys[str(name)] = id
                
            elif parm == 'Movie':
                movie = self.data.search_movie(term)
                for id, title in movie.items():
                    self.window.leftbox.addItem(str(title))
                    self.keys[str(title)] = id
 
    def display(self):
        ''' Method for displaying movie informaion '''
        movie = self.window.leftbox.currentItem().text()
        movie_id = self.keys.get(movie)
        detail = self.data.get_movie(movie_id)

        frame = self.window.frame_container


        for index in reversed(range(frame.count())):
            widgettoremove = frame.itemAt(index).widget()
            frame.removeWidget(widgettoremove)
            widgettoremove.setParent(None)

        slider = self.window.scrollbar.verticalScrollBar()
        slider.setValue(0)

        Page(frame, detail)


if __name__ == '__main__':
    app = QApplication([])
    controller = Controller(Data(), Window())
    app.exec()